-------------------------------------------------------------------------------
--|> Bars
-- Manages the display of the bars and their elements representing detected players
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local InCombatLockdown = InCombatLockdown
local GetUnitName = GetUnitName
local GameFontNormal = GameFontNormal

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> GLOBALS
-------------------------------------------------------------------------------
weizPVP.Bars = {}
weizPVP.Buttons = {}
weizPVP.Icons = {}
weizPVP.TargetedIcon = nil

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
local lastRow = 1

--=> GENERATE BARS IN LIST FRAME
--> Creates a set of bars to be placed on the ListFrame
-------------------------------------------------------------------------------
function weizPVP:CreateBars()
    local barNumber = 1 --> Start naming @ #1

    -------------------------------------------------------------------------------
    --=> Create all displayable bars
    -------------------------------------------------------------------------------
    for k = 0, weizPVP.Options.Bars.MaxNumBars - 1 do
        --=> CREATE BAR
        -----------------------------------------------------------------------
        local barName = "weizPVP.StatusBar" .. tostring(barNumber)
        local row = CreateFrame("StatusBar", barName, weizPVP.ListFrame)
        row.Highlight = row:CreateTexture(nil, "ARTWORK", nil, 1)
        row.NameText = row:CreateFontString(nil, "ARTWORK", nil, 2)
        row.LevelText = row:CreateFontString(nil, "ARTWORK", GameFontNormal, 2)
        row.GuildText = row:CreateFontString(nil, "ARTWORK", GameFontNormal, 2)

        row.id = barNumber
        row.name = ""

        --=> BAR ICONS
        row.RoleIcon = row:CreateTexture(nil, "ARTWORK", nil, 2)
        row.DEADIcon = row:CreateTexture(nil, "ARTWORK", nil, 2)

        if barNumber == 1 then -->If  its's the first bar, just anchor it to the top
            row:SetPoint("TOPLEFT", weizPVP.ListFrame, "TOPLEFT")
        else --> Bars after 1 using an offset t spacee themsleves out
            local offsetY = weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing
            offsetY = -1 * (k * offsetY)
            row:SetPoint("TOPLEFT", weizPVP.ListFrame, "TOPLEFT", 0, offsetY)
        end

        --> Set properties
        row:SetHeight(weizPVP.Options.Bars.RowHeight)
        row:SetPoint("RIGHT", weizPVP.ScrollFrame, "RIGHT")
        row:SetPoint("LEFT", weizPVP.ScrollFrame, "LEFT")
        -- row:SetStatusBarTexture(SM:Fetch("statusbar", weizPVP.Options.Bars.Texture))
        row:SetStatusBarTexture("Interface\\addons\\weizPVP\\Textures\\bar-bg.tga")
        row:SetStatusBarColor(0, 0, 0, 0)
        row:SetMinMaxValues(0, 100)
        row:EnableMouse(true)
        row:SetValue(100)

        --> Set Background
        row.bg = row:CreateTexture(nil, "BACKGROUND")
        row.bg:SetTexture(SM:Fetch("statusbar", weizPVP.Options.Bars.BackgroundTexture))
        row.bg:SetAllPoints(true)
        row.bg:SetVertexColor(0, 0, 0, 0)

        --=> Role Icon
        -----------------------------------------------------------------------
        row.RoleIcon:SetPoint("LEFT", row, "LEFT", 2, 0)
        row.RoleIcon:SetSize(16, 16)
        row.RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\unknown.tga", false)

        --=> Name Text
        -----------------------------------------------------------------------
        row.NameText:SetFont(SM:Fetch("font", weizPVP.Options.Bars.NameFont), weizPVP.Options.Bars.NameFontSize, "OUTLINE")
        row.NameText:SetHeight(weizPVP.Options.Bars.NameFontSize)
        row.NameText:SetPoint("LEFT", row.RoleIcon, "RIGHT", 2, 0)
        row.NameText:SetDrawLayer("OVERLAY", 7)
        row.NameText:SetJustifyH("LEFT")
        row.NameText:SetJustifyV("MIDDLE")
        row.NameText:SetTextColor(1, 1, 1, 1)
        row.NameText:SetShadowColor(0.2, 0.2, 0.2, 0.4)
        row.NameText:SetShadowOffset(1, -1)

        --=> Level Text
        -----------------------------------------------------------------------
        row.LevelText:SetFont(SM:Fetch("font", weizPVP.Options.Bars.LevelFont), weizPVP.Options.Bars.LevelFontSize, "OUTLINE")
        row.LevelText:SetHeight(weizPVP.Options.Bars.LevelFontSize)
        row.LevelText:SetPoint("LEFT", row.NameText, "RIGHT", 2, -1)
        row.LevelText:SetJustifyH("LEFT")
        row.LevelText:SetTextColor(1, 1, 1, 1)
        row.LevelText:SetShadowColor(0, 0, 0, 0)

        --=> Guild Text
        -----------------------------------------------------------------------
        row.GuildText:SetFont(SM:Fetch("font", weizPVP.Options.Bars.GuildFont), weizPVP.Options.Bars.GuildFontSize, "OUTLINE")
        row.GuildText:SetHeight(weizPVP.Options.Bars.GuildFontSize)
        row.GuildText:SetMaxLines(1)
        row.GuildText:SetPoint("RIGHT", row, "RIGHT", -4, -1)
        row.GuildText:SetJustifyH("RIGHT")
        row.GuildText:SetJustifyV("MIDDLE")
        row.GuildText:SetTextColor(1, 1, 1, 1)
        row.GuildText:SetShadowColor(0, 0, 0, 0)

        --=> DEAD Icon
        -----------------------------------------------------------------------
        row.DEADIcon:SetPoint("LEFT", row.LevelText, "RIGHT", 4, 0)
        row.DEADIcon:SetSize(weizPVP.Options.Bars.RowHeight, weizPVP.Options.Bars.RowHeight)
        row.DEADIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\icon_dead.tga", false)
        row.DEADIcon:Hide()

        --=> SECURE BUTTON
        -------------------------------------------------------------------------------
        local secureButton = CreateFrame("Button", "weizPVP.SecureButton" .. barNumber, row, "weizPVPSecureButton")
        secureButton:SetPoint("TOPLEFT", row, "TOPLEFT")
        secureButton:SetPoint("BOTTOMRIGHT", row, "BOTTOMRIGHT")
        secureButton:SetAlpha(0)
        secureButton:Show()
        secureButton.id = barNumber

        --=> SECURE BUTTON TEXTURE
        -------------------------------------------------------------------------------
        row.Highlight:SetAllPoints()
        row.Highlight:SetColorTexture(1, 1, 1)
        row.Highlight:SetBlendMode("ADD")
        row.Highlight:SetAlpha(0)

        --> SECURE BUTTON: OnEnter
        -----------------------------------
        secureButton:SetScript(
            "OnEnter",
            function(self, event)
                row.Highlight:SetAlpha(0.15)
            end
        )
        --> SECURE BUTTON: OnLeave
        -----------------------------------
        secureButton:SetScript(
            "OnLeave",
            function(self, event)
                row.Highlight:SetAlpha(0)
            end
        )

        --=> Set Globals
        -----------------------------------------------------------------------
        weizPVP.Bars[barNumber] = row
        weizPVP.Buttons[barNumber] = secureButton

        --=> barNumber++
        -----------------------------------------------------------------------
        barNumber = barNumber + 1
    end

    --=> Target Icon
    -----------------------------------------------------------------------
    weizPVP.TargetedIcon = weizPVP.OuterIconsFrame:CreateTexture("ARTWORK")
    weizPVP.TargetedIcon:SetTexture("Interface\\addons\\weizPVP\\Icons\\target.tga")
    weizPVP.TargetedIcon:SetPoint("RIGHT", weizPVP.Bars[1], "LEFT", -5, 0)
    weizPVP.TargetedIcon:SetBlendMode("BLEND")
    weizPVP.TargetedIcon:SetWidth(16)
    weizPVP.TargetedIcon:SetHeight(16)
    weizPVP.TargetedIcon:SetAlpha(0)

    weizPVP:RefreshCurrentList()
end

-------------------------------------------------------------------------------
--=> Change Target Icon
-------------------------------------------------------------------------------

function weizPVP:ChangeTargetIcon()
    --> Icon
    if weizPVP.playersOnBars[GetUnitName("target", true)] then
        weizPVP.TargetedIcon:SetPoint("RIGHT", weizPVP.Bars[weizPVP.playersOnBars[GetUnitName("target", true)]], "LEFT", 0, 0)
        weizPVP.TargetedIcon:SetAlpha(1)

        weizPVP.Bars[lastRow]:SetStatusBarTexture("Interface\\addons\\weizPVP\\Textures\\bar-bg.tga")
        weizPVP.Bars[weizPVP.playersOnBars[GetUnitName("target", true)]]:SetStatusBarTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
        --
        lastRow = weizPVP.playersOnBars[GetUnitName("target", true)]
    else
        weizPVP.TargetedIcon:SetAlpha(0)
        weizPVP.Bars[lastRow]:SetStatusBarTexture("Interface\\addons\\weizPVP\\Textures\\bar-bg.tga")
    end
end

-------------------------------------------------------------------------------
--=> Row Clicked: Secure Button
-------------------------------------------------------------------------------
--> PRECLICK
function weizPVP:ButtonPreClick(self, button)
    local bar = weizPVP.Bars[self.id]
    local playerName = bar.name
    if playerName then
        if button == "LeftButton" then
            if not InCombatLockdown() then
                self:SetAttribute("macrotext", "/target " .. playerName)
            end
        end
    end
end

--> POSTCLICK
function weizPVP:ButtonPostClick(self, button)
    local bar = weizPVP.Bars[self.id]
    local playerName = bar.name
    if playerName then
        if button == "LeftButton" then
            local targetName = GetUnitName("target", true)
            if targetName ~= playerName and weizPVP.ActiveList[playerName] and weizPVP.PlayerActiveCache[playerName].Stealth ~= true and not InCombatLockdown() then
                weizPVP.InactiveList[playerName] = weizPVP.ActiveList[playerName]
                weizPVP.ActiveList[playerName] = nil
                weizPVP:SortNearbyList()
                weizPVP:RefreshCurrentList()
            end
        end
    end
end
