-------------------------------------------------------------------------------
--|> weizPVP
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitLevel = UnitLevel
local GetRealmName = GetRealmName
local GetUnitName = GetUnitName
local MAX_PLAYER_LEVEL_TABLE = MAX_PLAYER_LEVEL_TABLE
local GetExpansionLevel = GetExpansionLevel
local UnitFactionGroup = UnitFactionGroup
local wipe = wipe
local select = select
local UIParent = UIParent
local ReloadUI = ReloadUI
local LibStub = LibStub

-------------------------------------------------------------------------------
--|> INITIALIZE
-------------------------------------------------------------------------------
--> The start of processing; entry point.
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()
    --> LOAD ACE3 DB
    weizPVP.db = LibStub("AceDB-3.0"):New("weizPVP_DB")

    --> Load Database
    weizPVP:LoadDB()

    -------------------------------------------------------------------------------
    --=> Enviromental Variables
    --> Holds infomation about the player's stats, game settings, system settings, etc.
    -------------------------------------------------------------------------------
    --> Get Player Unit Info
    weizPVP.Player = {}
    weizPVP.Player.Level = UnitLevel("player") --> player's level
    weizPVP.Player.CurrentRealm = GetRealmName() --> player's realm name
    weizPVP.Player.Name = GetUnitName("player") --> player's char name
    weizPVP.Player.Faction = select(1, UnitFactionGroup("player")) --> Horde/Alliance
    weizPVP.MaxPlayerLevel = MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()]

    --> Get Display Info
    weizPVP.Display = {}
    weizPVP.Display.Width = GetScreenWidth()
    weizPVP.Display.Height = GetScreenHeight()
    weizPVP.Display.xFactor = (768 / weizPVP.Display.Width) * (weizPVP.Display.Width / weizPVP.Display.Height)
    weizPVP.Display.yFactor = 768 / weizPVP.Display.Height
    weizPVP.Display.UIScale = UIParent:GetScale()
    weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
    weizPVP.Display.CVarUiScale = GetCVar("uiScale")
    weizPVP.Display.CurrentFPS = GetFramerate()

    --> Init Static Data Tables
    weizPVP:CreateAbilityLevelDB()
    weizPVP:CreateSpecDB()

    --> Create Core Frames and Bars
    weizPVP:CreateCoreFrames()
    weizPVP:CreateBars()
    weizPVP:CreateStatusBar()

    --> Create Crosshair Frames
    -- weizPVP.EnableCrosshair()

    --> Create Stealth Alert Frame
    weizPVP:CreateStealthAlertFrame()

    --> Save Options
    weizPVP:ApplySavedOptions()
end

-------------------------------------------------------------------------------
--|> ON LOAD
-------------------------------------------------------------------------------
function weizPVP:OnLoad()
end

-------------------------------------------------------------------------------
--|> ON ENABLE
-------------------------------------------------------------------------------
function weizPVP:OnEnable()
    weizPVP.HeaderFrame:Show()
    weizPVP.CoreFrame:Show()
    weizPVP.FooterFrame:Show()
    weizPVP.Options.Enabled = true
    --> Timers
    weizPVP:ScheduleRepeatingTimer("OneSecondTimer", 1) --> 1 second timer used for periodic updating.
    weizPVP:ScheduleRepeatingTimer("SaveAddonData", 10) --> Saves data every 10 seconds

    --=> Registered Events
    -------------------------------------------------------------------------------
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")

    --> ZONE CHANGED
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")

    --> COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "DisplaySizeChanged")
    weizPVP:RegisterEvent("PLAYER_LOGIN", "DisplaySizeChanged")
end

-------------------------------------------------------------------------------
--|> ON DISABLE
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
    weizPVP.HeaderFrame:Hide()
    weizPVP.CoreFrame:Hide()
    weizPVP.FooterFrame:Hide()
    weizPVP.Options.Enabled = false
    --=> UnRegistered Events
    -------------------------------------------------------------------------------
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")

    --> ZONE CHANGED
    weizPVP:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:UnregisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")

    --> COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "DisplaySizeChanged")
    weizPVP:UnregisterEvent("PLAYER_LOGIN", "DisplaySizeChanged")
end

-------------------------------------------------------------------------------
--|> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------
--=> Load Database
--> Checks to see if DB exists, what version is being used, etc; loads.
-------------------------------------------------------------------------------
function weizPVP:LoadDB()
    --> Check for Player DB
    if not weizPVP.db.profile.PlayerDB then
        weizPVP.db.profile.PlayerDB = weizPVP.PlayerDB
    else
        weizPVP.PlayerDB = weizPVP.db.profile.PlayerDB
    end

    --> Check for existing Options and version
    if not weizPVP.db.profile.Options or weizPVP.db.profile.Options.DB_VERSION ~= weizPVP.Options.DB_VERSION then
        weizPVP:PrintAddonMessage("|cffff3838WARNING:|r The database has been updated to 1.1; All previous options will be set to defaults due to many changes.")

        --> REMOVAL OF OLDER SETTINGS DB
        if weizPVP.db.profile.Settings then
            weizPVP.db.profile.Settings = {}
        end

        local OptionsSet = weizPVP.SetDefaultOptionsTable()
        if OptionsSet then
            if weizPVP.db.profile.Options then
                weizPVP.Options = weizPVP.db.profile.Options
            else
                weizPVP.db.profile.Options = weizPVP.Options
            end
        else
            weizPVP:PrintAddonMessage("|cffff3838WARNING:|r Default Options were attempted to be set, but failed.")
        end
    else --> Saved Variable Options are value, apply to Options
        weizPVP.Options = weizPVP.db.profile.Options
    end

    --> Add in new options if they don't exist in saved variables
    if not weizPVP.Options.EnableBGs then
        weizPVP.Options.EnableBGs = false
    end
    if not weizPVP.Options.AudioAlerts then
        weizPVP.Options.AudioAlerts = {}
        weizPVP.Options.AudioAlerts.DetectedPlayerSound = false
    end
    if not weizPVP.Options.Crosshair.NameEnabled then
        weizPVP.Options.Crosshair.NameEnabled = false
    end
    if not weizPVP.Options.Crosshair.GuildEnabled then
        weizPVP.Options.Crosshair.GuildEnabled = false
    end

    --> Apply current settings
    weizPVP.SetStaticOptions()
    weizPVP:SaveAddonData()
end

--=> Save Addon Data
-------------------------------------------------------------------------------
function weizPVP:SaveAddonData()
    weizPVP.db.profile.PlayerDB = weizPVP.PlayerDB
    weizPVP.db.profile.Options = weizPVP.Options
end

--=> Reset All
-------------------------------------------------------------------------------
function weizPVP:ResetAll()
    if weizPVP.db.profile then
        wipe(weizPVP.db.profile)
        weizPVP.db.profile = {}
    end
    if weizPVP.PlayerDB then
        wipe(weizPVP.PlayerDB)
        weizPVP.PlayerDB = {}
    end
    if weizPVP.Options then
        wipe(weizPVP.Options)
        weizPVP.Options = {}
    end

    ReloadUI()
end
